"""This module is for the translation mapping data of the
following document:

Document file: int_and_number_arithmetic_operations.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Int and Number basic arithmetic operations": "# Int と Number クラスの基本的な計算制御",
    ##################################################
    "This page explains basic arithmetic operations of the `Int` and `Number` classes, like addition, multiplication, or incremental addition.": "このページでは`Int`や`Number`の各クラスの加算や乗算などの基本的な計算制御について説明します。",  # noqa
    ##################################################
    "## Common behaviors": "## 共通の挙動",
    ##################################################
    "You can calculate the `Int` and `Number` values with the Python built-in values, such as `int` or `float`\\, as follows:": "`Int`や`Number`の各値は以下のコード例のように`int`や`float`などのPythonビルトインの値と計算することができます:",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_1 = int_1 + 20\nassert int_1 == 30\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_1 = int_1 + 20\nassert int_1 == 30\n```",  # noqa
    ##################################################
    "Also, arithmetic operations with the same types (e.g., `Int` and `Int`) are supported:": "また、同じ型同士（例 : `Int`と`Int`同士など）での計算も同様にサポートしています:",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_2: ap.Int = ap.Int(20)\nint_1 = int_1 + int_2\nassert int_1 == 30\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_2: ap.Int = ap.Int(20)\nint_1 = int_1 + int_2\nassert int_1 == 30\n```",  # noqa
    ##################################################
    "Arithmetic operations are not supported if the left value is the Python built-in value. For instance, the following code raises an exception:": "計算で左側の値がPythonビルトインの値の場合はサポートしていません。例えば以下のコードではエラーとなります:",  # noqa
    ##################################################
    "```py\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\n\n# This will raise the error!\nint_1 = 20 + int_1\n```": "```py\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\n\n# This will raise the error!\nint_1 = 20 + int_1\n```",  # noqa
    ##################################################
    "```\nTypeError: unsupported operand type(s) for +: 'int' and 'Int'\n```": "```\nTypeError: unsupported operand type(s) for +: 'int' and 'Int'\n```",  # noqa
    ##################################################
    "## Addition": "## 加算",
    ##################################################
    "You can add values with the `+` operator.": "`+`のオペレーターを使って各値の加算処理を行うことができます。",
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_1 = int_1 + 20\nassert int_1 == 30\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_1 = int_1 + 20\nassert int_1 == 30\n```",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_2: ap.Int = ap.Int(20)\nint_1 = int_1 + int_2\nassert int_1 == 30\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_2: ap.Int = ap.Int(20)\nint_1 = int_1 + int_2\nassert int_1 == 30\n```",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10) + ap.Int(20)\nassert int_1 == 30\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10) + ap.Int(20)\nassert int_1 == 30\n```",  # noqa
    ##################################################
    "Also, you can use the `+=` operator.": "`+=`のオペレーターも同じように使用することができます。",
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_1 += 20\nassert int_1 == 30\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_1 += 20\nassert int_1 == 30\n```",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_2: ap.Int = ap.Int(20)\nint_1 += int_2\nassert int_1 == 30\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_2: ap.Int = ap.Int(20)\nint_1 += int_2\nassert int_1 == 30\n```",  # noqa
    ##################################################
    "## Subtraction": "## 減算",
    ##################################################
    "You can subtract values with the `-` operator.": "`-`のオペレーターを使って各値の減算を行うことができます。",
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(30)\nint_1 = int_1 - 10\nassert int_1 == 20\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(30)\nint_1 = int_1 - 10\nassert int_1 == 20\n```",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(30)\nint_2: ap.Int = ap.Int(20)\nint_1 = int_1 - int_2\nassert int_1 == 10\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(30)\nint_2: ap.Int = ap.Int(20)\nint_1 = int_1 - int_2\nassert int_1 == 10\n```",  # noqa
    ##################################################
    "Also, you can use the `-=` operator.": "`-=`のオペレーターも同様に使用することができます。",
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(50)\nint_1 -= 30\nassert int_1 == 20\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(50)\nint_1 -= 30\nassert int_1 == 20\n```",  # noqa
    ##################################################
    "## Multiplication": "## 乗算",
    ##################################################
    "You can multiply values with the `*` operator.": "`*`のオペレーターを使って各値を乗算することができます。",
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_1 = int_1 * 3\nassert int_1 == 30\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_1 = int_1 * 3\nassert int_1 == 30\n```",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_2: ap.Int = ap.Int(5)\nint_1 = int_1 * int_2\nassert int_1 == 50\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_2: ap.Int = ap.Int(5)\nint_1 = int_1 * int_2\nassert int_1 == 50\n```",  # noqa
    ##################################################
    "Also, you can use the `*=` operator.": "`*=`のオペレーターを使うこともできます。",
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_1 *= 3\nassert int_1 == 30\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_1 *= 3\nassert int_1 == 30\n```",  # noqa
    ##################################################
    "## Division": "## 除算",
    ##################################################
    "You can divide values with the `/` operator. A return value becomes a `Number` value, not an `Int`\\.": "`/`のオペレーターを使って各値の除算を行うことができます。返却値は`Int`の整数ではなく`Number`の浮動小数点数になります。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nnumber_1: ap.Number = int_1 / 4\nassert number_1 == 2.5\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nnumber_1: ap.Number = int_1 / 4\nassert number_1 == 2.5\n```",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_2: ap.Int = ap.Int(4)\nnumber_1: ap.Number = int_1 / int_2\nassert number_1 == 2.5\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_2: ap.Int = ap.Int(4)\nnumber_1: ap.Number = int_1 / int_2\nassert number_1 == 2.5\n```",  # noqa
    ##################################################
    "Also, you can use the `/=` operator.": "`/=`のオペレーターを使うこともできます。",
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nnumber_1: ap.Number = ap.Number(10)\nnumber_1 /= 4\nassert number_1 == 2.5\n```": "```py\n# runnable\nimport apysc as ap\n\nnumber_1: ap.Number = ap.Number(10)\nnumber_1 /= 4\nassert number_1 == 2.5\n```",  # noqa
    ##################################################
    "## Floor division": "## 切り捨て除算",
    ##################################################
    "You can divide and floor values with the `//` operator. A return value becomes an `Int` value, not a `Number`\\.": "`//`のオペレーターで除算と浮動小数点数の切り捨てを行うことができます。返却値は`Number`型の浮動小数点数ではなく`Int`の整数となります。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_1 = int_1 // 4\nassert int_1 == 2\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_1 = int_1 // 4\nassert int_1 == 2\n```",  # noqa
    ##################################################
    "## Modulo": "## 剰余",
    ##################################################
    "You can use the modulo operation with the `%` operator.": "`%`のオペレーターを使って剰余の計算を行うことができます。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_2: int = int_1 % 3\nassert int_2 == 1\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_2: int = int_1 % 3\nassert int_2 == 1\n```",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nnumber_1: ap.Number = ap.Number(10.5)\nnumber_2: ap.Number = number_1 % 3\nassert number_2 == 1.5\n```": "```py\n# runnable\nimport apysc as ap\n\nnumber_1: ap.Number = ap.Number(10.5)\nnumber_2: ap.Number = number_1 % 3\nassert number_2 == 1.5\n```",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: path_line_to.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# PathLineTo class": "# PathLineTo クラス",
    ##################################################
    "This page explains the `PathLineTo` class.": "このページでは`PathLineTo`クラスについて説明します。",
    ##################################################
    "## What class is this?": "## クラス概要",
    ##################################################
    "The `PathLineTo` class is the class to set a new line from the current position on a path.": "`PathLineTo`クラスは現在設定されている座標位置から新たな線のパスを描画します。",  # noqa
    ##################################################
    "Mainly, the `Path` class constructor or `draw_path` interfaces use this setting.": "主にこの設定は`Path`クラスのコンストラクタと`draw_path`メソッドのインターフェイスで使用されます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `PathLineTo` class constructor requires the `x` and `y` arguments.": "`PathLineTo`クラスのコンストラクタでは`x`と`y`の引数指定が必要となります。",  # noqa
    ##################################################
    "The `Path` class constructor or `draw_path` interfaces' `path_data_list` argument requires its instance.": "`Path`クラスのコンストラクタもしくは`draw_path`メソッドのインターフェイスの`path_data_list`引数でそのインスタンスが必要とされます。",  # noqa
    ##################################################
    "The following example sets the line drawing from x=50 and y=50 to x=150 and y=50 with the `PathLineTo` instance:": "以下のコード例ではx=50, y=50の位置からx=150, y=50の位置に向けて`PathLineTo`のインスタンスを設定して線の描画設定を行っています:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=200, stage_height=100, stage_elem_id="stage"\n)\npath: ap.Path = ap.Path(\n    path_data_list=[\n        ap.PathMoveTo(x=50, y=50),\n        ap.PathLineTo(x=150, y=50),\n    ],\n    line_color="#0af",\n    line_thickness=5,\n)\n\nap.save_overall_html(dest_dir_path="path_line_to_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=200, stage_height=100, stage_elem_id="stage"\n)\npath: ap.Path = ap.Path(\n    path_data_list=[\n        ap.PathMoveTo(x=50, y=50),\n        ap.PathLineTo(x=150, y=50),\n    ],\n    line_color="#0af",\n    line_thickness=5,\n)\n\nap.save_overall_html(dest_dir_path="path_line_to_basic_usage/")\n```',  # noqa
    ##################################################
    "## Relative position setting": "## 相対座標設定",
    ##################################################
    "The constructor's `relative` optional argument changes its behavior.": "コンストラクタの`relative`のオプション引数はその挙動を変更します。",  # noqa
    ##################################################
    "For example, if you set True to its argument, coordinates become relative.": "例として、もしその引数にTrueを指定した場合座標は相対座標として設定されます。",  # noqa
    ##################################################
    "The default setting is False, and it becomes absolute.": "デフォルト値はFalseとなっており、この設定では絶対座標として扱われます。",  # noqa
    ##################################################
    "The following example sets the relative setting and draws the line to the 50px under position from the current position:": "以下のコード例ではrelativeの設定を行い、現在の位置から50px下の位置に向けて線の描画を行っています:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=100, stage_height=150, stage_elem_id="stage"\n)\npath: ap.Path = ap.Path(\n    path_data_list=[\n        ap.PathMoveTo(x=50, y=50),\n        ap.PathLineTo(x=0, y=50, relative=True),\n    ],\n    line_color="#0af",\n    line_thickness=5,\n)\n\nap.save_overall_html(dest_dir_path="path_line_to_relative/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=100, stage_height=150, stage_elem_id="stage"\n)\npath: ap.Path = ap.Path(\n    path_data_list=[\n        ap.PathMoveTo(x=50, y=50),\n        ap.PathLineTo(x=0, y=50, relative=True),\n    ],\n    line_color="#0af",\n    line_thickness=5,\n)\n\nap.save_overall_html(dest_dir_path="path_line_to_relative/")\n```',  # noqa
    ##################################################
    "## PathLineTo class constructor API": "## PathLineTo クラスのコンストラクタのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Path data class for the SVG `line to` (L).<hr>": "SVGの特定座標に対する線（L）の描画のためのパスデータのクラスです。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `x`: Int or int": "- `x`: Int or int",
    ##################################################
    "  - X-coordinate of the destination point.": "  - 終点のX座標。",
    ##################################################
    "- `y`: Int or int": "- `y`: Int or int",
    ##################################################
    "  - Y-coordinate of the destination point.": "  - 終点のY座標。",
    ##################################################
    "- `relative`: bool or Boolean, default False": "- `relative`: bool or Boolean, default False",  # noqa
    ##################################################
    "  - A boolean value indicates whether the path coordinates are relative or not (absolute).": "  - パスの座標が相対座標として扱うかもしくは絶対座標として扱うかどうかの真偽値。",  # noqa
    ##################################################
    "- `variable_name_suffix`: str, default ''": "- `variable_name_suffix`: str, default ''",  # noqa
    ##################################################
    "  - A JavaScript variable name suffix string. This setting is sometimes useful for JavaScript's debugging.": "  - JavaScript上の変数のサフィックスの設定です。この設定はJavaScriptのデバッグ時に役立つことがあります。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color="#fff", thickness=3)\n>>> path: ap.Path = sprite.graphics.draw_path(\n...     path_data_list=[\n...         ap.PathMoveTo(x=0, y=50),\n...         ap.PathLineTo(x=50, y=50),\n...     ]\n... )\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color="#fff", thickness=3)\n>>> path: ap.Path = sprite.graphics.draw_path(\n...     path_data_list=[\n...         ap.PathMoveTo(x=0, y=50),\n...         ap.PathLineTo(x=50, y=50),\n...     ]\n... )\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [Path class](https://simon-ritchie.github.io/apysc/en/path.html)": "- [Path クラス](https://simon-ritchie.github.io/apysc/jp/jp_path.html)",  # noqa
    ##################################################
    "- [Graphics draw_path interface](https://simon-ritchie.github.io/apysc/en/graphics_draw_path.html)": "- [Graphics クラスの draw_path インターフェイス](https://simon-ritchie.github.io/apysc/jp/jp_graphics_draw_path.html)",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: assert_arrays_equal_and_arrays_not_equal.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# assert_arrays_equal and assert_arrays_not_equal interfaces": "# assert_arrays_equal と assert_arrays_not_equal インターフェイス",  # noqa
    ##################################################
    "This page explains the `assert_arrays_equal` and `assert_arrays_not_equal` function interfaces.": "このページでは`assert_arrays_equal`と`assert_arrays_not_equal`関数の各インターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interfaces are these?": "## 各インターフェイスの概要",
    ##################################################
    "The `assert_arrays_equal` function interface asserts that the two arrays are equal. Conversely, the `assert_arrays_not_equal` function interface asserts that the two arrays are not equal.": "`assert_arrays_equal`関数のインターフェイスは2つの配列の値が一致していることをチェックします。逆に`assert_arrays_not_equal`関数は2つの配列の値が一致していないことをチェックします。",  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [JavaScript assertion interface basic behavior](assertion_basic_behavior.md)": "- [JavaScriptの各アサーションのインターフェイスの基本的な挙動](jp_assertion_basic_behavior.md)",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "Both of the `assert_arrays_equal` and `assert_arrays_not_equal` interfaces require the `left` and `right` arguments. The `msg` argument is optional.": "`assert_arrays_equal`と`assert_arrays_not_equal`のインターフェイスは共に`left`と`right`という2つの引数を必要とします。`msg`引数は省略可です。",  # noqa
    ##################################################
    "The arguments accept a Python built-in `list` value and `Array` value.": "引数にはPythonビルトインの`list`の値もしくはapyscの`Array`の値を指定することができます。",  # noqa
    ##################################################
    "The following example (`assert_arrays_equal` and values are equal) passes:": "以下のコードで例では`assert_arrays_equal`関数を使って値が一致していることを確認しています:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color="#333", stage_elem_id="stage"\n)\n\narr_1: ap.Array = ap.Array([1, 2, 3])\nap.assert_arrays_equal(left=[1, 2, 3], right=arr_1, msg="Values are not equal!")\n\nap.save_overall_html(dest_dir_path="assert_arrays_equal_basic_usage_1/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color="#333", stage_elem_id="stage"\n)\n\narr_1: ap.Array = ap.Array([1, 2, 3])\nap.assert_arrays_equal(left=[1, 2, 3], right=arr_1, msg="Values are not equal!")\n\nap.save_overall_html(dest_dir_path="assert_arrays_equal_basic_usage_1/")\n```',  # noqa
    ##################################################
    "```\n[assert_arrays_equal]\nLeft value: [1, 2, 3] right value: arr_2\n```": "```\n[assert_arrays_equal]\nLeft value: [1, 2, 3] right value: arr_2\n```",  # noqa
    ##################################################
    "The following example (`assert_arrays_equal` and values are not equal) fails:": "以下のコード例では`assert_arrays_equal`関数を使って値が一致しておらずチェックが失敗していることを確認しています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color="#333", stage_elem_id="stage"\n)\n\narr_1: ap.Array = ap.Array([1, 2, 3])\nap.assert_arrays_equal(left=[1, 2], right=arr_1, msg="Values are not equal!")\n\nap.save_overall_html(dest_dir_path="assert_arrays_equal_basic_usage_2/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color="#333", stage_elem_id="stage"\n)\n\narr_1: ap.Array = ap.Array([1, 2, 3])\nap.assert_arrays_equal(left=[1, 2], right=arr_1, msg="Values are not equal!")\n\nap.save_overall_html(dest_dir_path="assert_arrays_equal_basic_usage_2/")\n```',  # noqa
    ##################################################
    "```\n[assert_arrays_equal]\nLeft value: [1, 2] right value: arr_2\n...\nAssertion failed: Values are not equal!\n```": "```\n[assert_arrays_equal]\nLeft value: [1, 2] right value: arr_2\n...\nAssertion failed: Values are not equal!\n```",  # noqa
    ##################################################
    "The following example (`assert_arrays_not_equal` and values are not equal) passes:": "以下のコード例では`assert_arrays_not_equal`関数を使って値が一致していないためチェックを通っていることを確認しています:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color="#333", stage_elem_id="stage"\n)\n\narr_1: ap.Array = ap.Array([1, 2, 3])\nap.assert_arrays_not_equal(left=[1, 2], right=arr_1, msg="Values are equal!")\n\nap.save_overall_html(dest_dir_path="assert_arrays_not_equal_basic_usage_1/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color="#333", stage_elem_id="stage"\n)\n\narr_1: ap.Array = ap.Array([1, 2, 3])\nap.assert_arrays_not_equal(left=[1, 2], right=arr_1, msg="Values are equal!")\n\nap.save_overall_html(dest_dir_path="assert_arrays_not_equal_basic_usage_1/")\n```',  # noqa
    ##################################################
    "## Notes for the assert_equal and assert_not_equal interfaces": "## assert_equal と assert_not_equal の各インターフェイスにおける特記事項",  # noqa
    ##################################################
    "If an `Array` value is specified to the `assert_equal` or `assert_not_equal` interface's values, then the `assert_arrays_equal` or `assert_arrays_not_equal` interfaces will be called instead of the `assert_equal` or `assert_not_equal` interfaces automatically.": "もしも`assert_equal`もしくは`assert_not_equal`のインターフェイスに`Array`の値が指定された場合、自動的にそれらのインターフェイスの代わりに`assert_arrays_equal`と`assert_arrays_not_equal`のインターフェイスが使用されます。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color="#333", stage_elem_id="stage"\n)\n\narr_1: ap.Array = ap.Array([1, 2, 3, 4, 5])\nap.assert_equal(left=[1, 2, 3, 4, 5], right=arr_1, msg="Values are equal!")\n\nap.save_overall_html(dest_dir_path="assert_arrays_equal_notes_for_the_assert_equal/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color="#333", stage_elem_id="stage"\n)\n\narr_1: ap.Array = ap.Array([1, 2, 3, 4, 5])\nap.assert_equal(left=[1, 2, 3, 4, 5], right=arr_1, msg="Values are equal!")\n\nap.save_overall_html(dest_dir_path="assert_arrays_equal_notes_for_the_assert_equal/")\n```',  # noqa
    ##################################################
    "```\n[assert_arrays_equal]\nLeft value: [1, 2, 3, 4, 5] right value: arr_2\n```": "```\n[assert_arrays_equal]\nLeft value: [1, 2, 3, 4, 5] right value: arr_2\n```",  # noqa
    ##################################################
    "## assert_arrays_equal API": "## assert_arrays_equal API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "JavaScript assertion interface for the Array values equal condition.<hr>": "配列の値の等値条件のためのJavaScript上のアサーションのインターフェイスです。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `left`: *": "- `left`: *",
    ##################################################
    "  - Left-side value to compare.": "  - 比較用の左辺の値。",
    ##################################################
    "- `right`: *": "- `right`: *",
    ##################################################
    "  - Right-side value to compare.": "  - 比較用の右辺の値。",
    ##################################################
    "- `msg`: str, optional": "- `msg`: str, optional",
    ##################################################
    "  - Message to display when assertion failed.": "  - チェックに失敗した際に表示するメッセージ。",
    ##################################################
    "- `outer_frames_index_adjustment`: int, optional": "- `outer_frames_index_adjustment`: int, optional",  # noqa
    ##################################################
    "  - The trace's outer frames index adjustment setting. This function uses this argument to adjust the caller's information. Also, this function only uses this argument in internal logic.": "  - trace関数の関数外の参照するフレームのインデックスの調整値です。この引数は呼び出し元の情報の位置を調整するのに使用されます。また、この引数は内部のロジックでのみ使用されるため通常は設定する必要はありません。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Notes]**": "**[特記事項]**",
    ##################################################
    "This interface is used instead of assert_equal for Array class comparison (JavaScript can not compare arrays directly, like a Python, for example, `[1, 2] === [1, 2]` becomes false).<hr>": "このインターフェイスは`Array`クラスの値の比較時には`assert_equal`インターフェイスの代わりに使用されます（JavaScript上ではPythonのリストのように直接配列の比較が行えないため代わりにこのインターフェイスが使用されます）。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> arr_1: ap.Array = ap.Array([1, 2, 3])\n>>> arr_2: ap.Array = ap.Array([1, 2, 3])\n>>> ap.assert_arrays_equal(arr_1, arr_2)\n```": "```py\n>>> import apysc as ap\n>>> arr_1: ap.Array = ap.Array([1, 2, 3])\n>>> arr_2: ap.Array = ap.Array([1, 2, 3])\n>>> ap.assert_arrays_equal(arr_1, arr_2)\n```",  # noqa
    ##################################################
    "## assert_arrays_not_equal API": "## assert_arrays_not_equal API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "JavaScript assertion interface for the Array values not equal condition.<hr>": "配列の値の非等値条件のためのJavaScript上のアサーションのインターフェイスです。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `left`: *": "- `left`: *",
    ##################################################
    "  - Left-side value to compare.": "  - 比較用の左辺の値。",
    ##################################################
    "- `right`: *": "- `right`: *",
    ##################################################
    "  - Right-side value to compare.": "  - 比較用の右辺の値。",
    ##################################################
    "- `msg`: str, optional": "- `msg`: str, optional",
    ##################################################
    "  - Message to display when assertion failed.": "  - チェックに失敗した際に表示するメッセージ。",
    ##################################################
    "- `outer_frames_index_adjustment`: int, optional": "- `outer_frames_index_adjustment`: int, optional",  # noqa
    ##################################################
    "  - The trace's outer frames index adjustment setting. This function uses this argument to adjust the caller's information. Also, this function only uses this argument in internal logic.": "  - trace関数の関数外の参照するフレームのインデックスの調整値です。この引数は呼び出し元の情報の位置を調整するのに使用されます。また、この引数は内部のロジックでのみ使用されるため通常は設定する必要はありません。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Notes]**": "**[特記事項]**",
    ##################################################
    "This interface is used instead of assert_not_equal for Array class comparison (JavaScript can not compare arrays directly, like a Python, for example, `[1, 2] === [1, 2]` becomes false).<hr>": "このインターフェイスは`Array`クラスの値の比較時には`assert_not_equal`インターフェイスの代わりに使用されます（JavaScript上ではPythonのリストのように直接配列の比較が行えないため代わりにこのインターフェイスが使用されます）。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> arr_1: ap.Array = ap.Array([1, 2, 3])\n>>> arr_2: ap.Array = ap.Array([4, 5, 6])\n>>> ap.assert_arrays_not_equal(arr_1, arr_2)\n```": "```py\n>>> import apysc as ap\n>>> arr_1: ap.Array = ap.Array([1, 2, 3])\n>>> arr_2: ap.Array = ap.Array([4, 5, 6])\n>>> ap.assert_arrays_not_equal(arr_1, arr_2)\n```",  # noqa
}



"""Type related common implementations.

Mainly following interface is defined:

- is_number
    - Get a boolean value whether a specified value is a Number value.
- is_float_or_number
    - Get a boolean value whether a specified value is
    a float or Number value.
- is_bool
    - Get a boolean value whether a specified value is bool
    or Boolean value.
- is_same_class_instance
    - Get a boolean value whether a specified class and
    instance's class are the same or not.
- is_immutable_type
    - Get a boolean value whether a specified value is an immutable
    type or not.
"""

from typing import Any
from typing import Tuple
from typing import Type


def is_number(*, value: Any) -> bool:
    """
    Get a boolean value whether a specified value is a Number value.

    Parameters
    ----------
    value : *
        Any value to check.

    Returns
    -------
    result : bool
        If a Number value is specified, this interface returns True.
    """
    import apysc as ap

    if isinstance(value, ap.Number):
        return True
    return False


def is_float_or_number(*, value: Any) -> bool:
    """
    Get a boolean value whether a specified value is
    a float or Number value.

    Parameters
    ----------
    value : *
        Any value to check.

    Returns
    -------
    result : bool
        If float or Number value is specified, this interface
        returns True.
    """
    if isinstance(value, float):
        return True
    if is_number(value=value):
        return True
    return False


def is_bool(*, value: Any) -> bool:
    """
    Get a boolean value whether a specified value is bool
    or Boolean value.

    Parameters
    ----------
    value : *
        Any value to check.

    Returns
    -------
    result : bool
        If bool or Boolean value is specified, this interface
        returns True.
    """
    if is_same_class_instance(class_=bool, instance=value):
        return True
    import apysc as ap

    if isinstance(value, ap.Boolean):
        return True
    return False


def is_same_class_instance(*, class_: Type, instance: Any) -> bool:
    """
    Get a boolean value whether a specified class and
    instance's class are the same or not.

    Notes
    -----
    If an instance is a subclass of the `cls` argument
    (differ from `isinstance`), this interface returns False.

    Parameters
    ----------
    class_ : Type
        Expected class.
    instance : *
        Intance to check it's class.

    Returns
    -------
    result : bool
        If a specified class and instance's class are the same,
        this interface returns True.
    """
    instance_type: Type = type(instance)  # type: ignore
    if instance_type == class_:
        return True
    return False


def is_immutable_type(*, value: Any) -> bool:
    """
    Get a boolean value whether a specified value is an immutable
    type or not.

    Notes
    -----
    apysc's value types, such as the `Int`, are checked
    as immutable since these js types are immutable.

    Parameters
    ----------
    value : Any
        Target value to check.

    Returns
    -------
    result : bool
        This interface checks the apysc value types as
        immutable to match the JavaScript behavior.
    """
    import apysc as ap

    immutable_types: Tuple = (
        int,
        float,
        bool,
        str,
        complex,
        tuple,
        range,
        bytes,
        ap.Int,
        ap.Number,
        ap.String,
        ap.Boolean,
    )
    if isinstance(value, immutable_types):
        return True
    return False



"""This module is for the translation mapping data of the
following document:

Document file: timer_repeat_count.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Timer class repeat_count setting": "# Timer クラスの repeat_count 設定",
    ##################################################
    "This page explains the `Timer` class `repeat_count` argument setting.": "このページでは`Timer`クラスの`repeat_count`引数の設定について説明します。",  # noqa
    ##################################################
    "## What argument is this?": "## 引数の概要",
    ##################################################
    "The `repeat_count` argument setting determines the max handler calling number. For example, if you specify the 10 value, a timer calls the handler 10 times and stops.": "`repeat_count`引数の設定ではハンドラが呼ばれる最大数を設定できます。例えば、もし10を指定した場合タイマーは10回ハンドラを呼び出した後に停止します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "You can set the `repeat_count` parameter at the `Timer` constructor. The following example sets the timer with the 100 times `repeat_count` value when clicking the rectangle.": "`Timer`クラスのコンストラクタにて`repeat_count`引数のパラメーターを設定することができます。以下のコード例では四角をクリックした際に`repeat_count`の値が100のタイマーを設定しています。",  # noqa
    ##################################################
    "If the timer moves the rectangle 100 times (100-pixels to the right), the timer stops.": "もしタイマーがハンドラ内で四角を100回分動かした場合（100px分右に動いた場合）タイマーは停止します。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_rectangle_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that a rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    options_: _RectOptions = {"rectangle": e.this}\n    timer: ap.Timer = ap.Timer(\n        handler=on_timer, delay=16, repeat_count=100, options=options_\n    )\n    timer.start()\n    e.this.unbind_click(handler=on_rectangle_click)\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that a timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.x += 1\n\n\nap.Stage(\n    stage_width=250, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_rectangle_click)\n\nap.save_overall_html(dest_dir_path="timer_repeat_count_basic_usage/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_rectangle_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that a rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    options_: _RectOptions = {"rectangle": e.this}\n    timer: ap.Timer = ap.Timer(\n        handler=on_timer, delay=16, repeat_count=100, options=options_\n    )\n    timer.start()\n    e.this.unbind_click(handler=on_rectangle_click)\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that a timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.x += 1\n\n\nap.Stage(\n    stage_width=250, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_rectangle_click)\n\nap.save_overall_html(dest_dir_path="timer_repeat_count_basic_usage/")\n```',  # noqa
    ##################################################
    "## Timer constructor API": "## Timer クラスのコンストラクタのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Timer class to handle function calling at regular intervals.<hr>": "一定間隔ごとにハンドラの関数を実行するためのタイマーのクラスです。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `handler`: _Handler": "- `handler`: _Handler",
    ##################################################
    "  - A handler would be called at regular intervals.": "  - 一定間隔ごとに呼ばれる関数もしくはメソッドのハンドラ。",  # noqa
    ##################################################
    "- `delay`: Int or int or Number or float or FPS": "- `delay`: Int or int or Number or float or FPS",  # noqa
    ##################################################
    "  - A delay between each `Handler` calling in a millisecond or FPS value. If an `FPS` value is specified, this value becomes a millisecond calculated with that FPS value (e.g., if the `FPS_60` value is specified, then `delay` becomes 16.6666667).": "  - ハンドラの実行間隔となるミリ秒もしくはFPSのenumの値。もし`FPS`の値が指定された場合、FPSに応じて計算されたミリ秒が設定されます（例えば、もし`FPS_60`が指定されていれば`delay`の値は16.6666667ミリ秒相当になります。）。",  # noqa
    ##################################################
    "- `repeat_count`: Int or int": "- `repeat_count`: Int or int",
    ##################################################
    "  - Max count of a `Handler`'s calling. A timer stops if the `Handler`'s calling count has reached this value. If 0 is specified, then a timer loops forever.": "  - ハンドラの実行回数の上限値。ハンドラの実行回数がこの値に到達した場合タイマーは停止します。もし0が指定された場合にはタイマーは停止しなくなります。",  # noqa
    ##################################################
    "- `options`: dict or None, default None": "- `options`: dict or None, default None",  # noqa
    ##################################################
    "  - Optional arguments dictionary to pass a `Handler` callable.": "  - ハンドラの関数もしくはメソッドへ渡すオプションとしての各パラメーターを格納した辞書。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.x += 1\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> _ = ap.Timer(on_timer, delay=ap.FPS.FPS_60, options=options).start()\n```': '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.x += 1\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> _ = ap.Timer(on_timer, delay=ap.FPS.FPS_60, options=options).start()\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [Timer](https://simon-ritchie.github.io/apysc/en/timer.html)": "- [Timer クラス](https://simon-ritchie.github.io/apysc/jp/jp_timer.html)",  # noqa
    ##################################################
    "- [TimerEvent class](https://simon-ritchie.github.io/apysc/en/timer_event.html)": "- [TimerEvent クラス](https://simon-ritchie.github.io/apysc/jp/jp_timer_event.html)",  # noqa
    ##################################################
    "- [Timer class delay setting](https://simon-ritchie.github.io/apysc/en/timer_delay.html)": "- [Timer クラスの delay 設定](https://simon-ritchie.github.io/apysc/jp/jp_timer_delay.html)",  # noqa
    ##################################################
    "- [FPS enum](https://simon-ritchie.github.io/apysc/en/fps.html)": "- [FPS の enum](https://simon-ritchie.github.io/apysc/jp/jp_fps.html)",  # noqa
    ##################################################
    "- [About the handler options' type](https://simon-ritchie.github.io/apysc/en/about_handler_options_type.html)": "- [ハンドラのoptions引数の型について](https://simon-ritchie.github.io/apysc/jp/jp_about_handler_options_type.html)",  # noqa
    ##################################################
    "## repeat_count property API": "## repeat_count 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a max count value of a handler's calling.<hr>": "ハンドラが呼ばれる最大数を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `repeat_count`: Int": "- `repeat_count`: Int",
    ##################################################
    "  - Max count of a handler's calling. If this value is 0, then a timer loop forever.": "  - ハンドラの呼び出しの上限回数。もし0が指定された場合、タイマーはずっと実行され続けます（ハンドラを呼び続けます）。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> def on_timer(e: ap.TimerEvent, options: dict) -> None:\n...     pass\n>>> timer: ap.Timer = ap.Timer(on_timer, delay=33.3, repeat_count=50)\n>>> timer.repeat_count\nInt(50)\n```": "```py\n>>> import apysc as ap\n>>> def on_timer(e: ap.TimerEvent, options: dict) -> None:\n...     pass\n>>> timer: ap.Timer = ap.Timer(on_timer, delay=33.3, repeat_count=50)\n>>> timer.repeat_count\nInt(50)\n```",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: assert_defined_and_undefined.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# assert_defined and assert_undefined interfaces": "# assert_defined と assert_undefined のインターフェイス",  # noqa
    ##################################################
    "This page explains the `assert_defined` and `assert_undefined` function interfaces.": "このページでは`assert_defined`と`assert_undefined`の関数の各インターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interfaces are these?": "## 各インターフェイスの概要",
    ##################################################
    "The `assert_defined` function interface asserts specified value is defined (initialized). Conversely, the `assert_undefined` function interface asserts specified value is undefined (not initialized or deleted).": "`assert_defined`関数のインターフェイスは指定された値が定義済みかどうか（初期化されているか）をチェックします。逆に`assert_undefined`関数は指定された値が定義されていない（`undefine`）状態や削除済みかどうかなどをチェックします。",  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [JavaScript assertion interface basic behavior](assertion_basic_behavior.md)": "- [JavaScriptの各アサーションのインターフェイスの基本的な挙動](jp_assertion_basic_behavior.md)",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "Both of the `assert_defined` and `assert_undefined` interfaces requires `value` argument and `msg` argument is optional.": "`assert_defined`と`assert_undefined`の各インターフェイスは共に`value`引数が必要になります。`msg`引数は省略可です。",  # noqa
    ##################################################
    "The following assertion example (`assert_defined` and initialized value) passes:": "以下のコード例では初期化されている値に対して`assert_defined`関数でチェックを行っています（チェックを通ります）:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color="#333", stage_elem_id="stage"\n)\n\nint_val: ap.Int = ap.Int(10)\nap.assert_defined(value=int_val, msg="Value is not defined!")\n\nap.save_overall_html(dest_dir_path="assert_defined_basic_usage_1/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color="#333", stage_elem_id="stage"\n)\n\nint_val: ap.Int = ap.Int(10)\nap.assert_defined(value=int_val, msg="Value is not defined!")\n\nap.save_overall_html(dest_dir_path="assert_defined_basic_usage_1/")\n```',  # noqa
    ##################################################
    "```\n[assert_defined]\nRight-side variable name: i_11\nLeft value: other than undefined right value: 10\n```": "```\n[assert_defined]\nRight-side variable name: i_11\nLeft value: other than undefined right value: 10\n```",  # noqa
    ##################################################
    "The following assertion example (`assert_defined` and the deleted value) fails:": "以下のコード例では削除の済みの値に対して`assert_defined`関数でチェックをしています（チェックは失敗します）:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color="#333", stage_elem_id="stage"\n)\n\nint_val: ap.Int = ap.Int(10)\nap.append_js_expression(expression=f"{int_val.variable_name} = undefined;")\nap.assert_defined(value=int_val, msg="Value is not defined!")\n\nap.save_overall_html(dest_dir_path="assert_defined_basic_usage_2/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color="#333", stage_elem_id="stage"\n)\n\nint_val: ap.Int = ap.Int(10)\nap.append_js_expression(expression=f"{int_val.variable_name} = undefined;")\nap.assert_defined(value=int_val, msg="Value is not defined!")\n\nap.save_overall_html(dest_dir_path="assert_defined_basic_usage_2/")\n```',  # noqa
    ##################################################
    "```\n[assert_defined]\nRight-side variable name: i_11\nLeft value: other than undefined right value: undefined\n...\nAssertion failed: Value is not defined!\n```": "```\n[assert_defined]\nRight-side variable name: i_11\nLeft value: other than undefined right value: undefined\n...\nAssertion failed: Value is not defined!\n```",  # noqa
    ##################################################
    "The following assertion example (`assert_undefined` and the deleted value) passes:": "以下のコード例では削除済みの値に対して`assert_undefined`関数でチェックを行っています（チェックは通ります）:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color="#333", stage_elem_id="stage"\n)\n\nint_val: ap.Int = ap.Int(10)\nap.append_js_expression(expression=f"{int_val.variable_name} = undefined;")\nap.assert_undefined(value=int_val, msg="Value is defined!")\n\nap.save_overall_html(dest_dir_path="assert_undefined_basic_usage_1/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color="#333", stage_elem_id="stage"\n)\n\nint_val: ap.Int = ap.Int(10)\nap.append_js_expression(expression=f"{int_val.variable_name} = undefined;")\nap.assert_undefined(value=int_val, msg="Value is defined!")\n\nap.save_overall_html(dest_dir_path="assert_undefined_basic_usage_1/")\n```',  # noqa
    ##################################################
    "```\n[assert_undefined]\nRight-side variable name: i_11\nLeft value: undefined right value: undefined\n```": "```\n[assert_undefined]\nRight-side variable name: i_11\nLeft value: undefined right value: undefined\n```",  # noqa
    ##################################################
    "## assert_defined API": "## assert_defined API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "JavaScript assertion interface for the defined (not undefined) value condition.<hr>": "定義済みの値かどうかの条件のためのJavaScript上のアサーションのインターフェイスです。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `value`: *": "- `value`: *",
    ##################################################
    "  - Target value to check.": "  - チェック対象の値。",
    ##################################################
    "- `msg`: str, optional": "- `msg`: str, optional",
    ##################################################
    "  - Message to display when assertion failed.": "  - チェックに失敗した際に表示するメッセージ。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> int_val: ap.Int = ap.Int(10)\n>>> ap.assert_defined(int_val)\n```": "```py\n>>> import apysc as ap\n>>> int_val: ap.Int = ap.Int(10)\n>>> ap.assert_defined(int_val)\n```",  # noqa
    ##################################################
    "## assert_undefined API": "## assert_undefined API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "JavaScript assertion interface for the undefined value condition.<hr>": "未定義の値かどうかの条件のためのJavaScript上のアサーションのインターフェイスです。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `value`: *": "- `value`: *",
    ##################################################
    "  - Target value to check.": "  - チェック対象の値。",
    ##################################################
    "- `msg`: str, optional": "- `msg`: str, optional",
    ##################################################
    "  - Message to display when assertion failed.": "  - チェックに失敗した際に表示するメッセージ。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> int_val: ap.Int = ap.Int(10)\n>>> ap.append_js_expression(expression=f"{int_val.variable_name} = undefined;")\n>>> ap.assert_undefined(int_val)\n```': '```py\n>>> import apysc as ap\n>>> int_val: ap.Int = ap.Int(10)\n>>> ap.append_js_expression(expression=f"{int_val.variable_name} = undefined;")\n>>> ap.assert_undefined(int_val)\n```',  # noqa
}



"""Class implementation for the scale_x_from_point mix-in.
"""

from typing import Any
from typing import Dict

from typing_extensions import final

from apysc._animation.animation_scale_x_from_point_mixin import (
    AnimationScaleXFromPointMixIn,
)
from apysc._display.set_lower_scale_limit_mixin import SetLowerScaleLimitMixIn
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.dictionary import Dictionary
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_suffix_attr_mixin import VariableNameSuffixAttrMixIn
from apysc._validation import arg_validation_decos


class ScaleXFromPointMixIn(
    VariableNameSuffixAttrMixIn,
    AnimationScaleXFromPointMixIn,
    SetLowerScaleLimitMixIn,
    RevertMixIn,
):

    _scale_x_from_point: Dictionary[str, Number]

    @final
    def _initialize_scale_x_from_point_if_not_initialized(self) -> None:
        """
        Initialize the `_scale_x_from_point` attribute if
        this instance does not initialize it yet.
        """
        if hasattr(self, "_scale_x_from_point"):
            return
        suffix: str = self._get_attr_variable_name_suffix(
            attr_identifier="scale_x_from_point"
        )
        self._scale_x_from_point = Dictionary(
            {},
            variable_name_suffix=suffix,
            skip_init_substitution_expression_appending=True,
        )

    @final
    @arg_validation_decos.is_apysc_integer(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def get_scale_x_from_point(self, *, x: Int) -> Number:
        """
        Get a scale-x value from the given x-coordinate.

        Parameters
        ----------
        x : Int
            X-coordinate.

        Returns
        -------
        scale_x : Number
            Scale-x value from the given x-coordinate.

        Notes
        -----
        The scale's minimum value is almost zero, and it does not become negative.

        References
        ----------
        - GraphicsBase scale_from_point interfaces
            - https://simon-ritchie.github.io/apysc/en/graphics_base_scale_from_point.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color="#0af")
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> x: ap.Int = ap.Int(100)
        >>> rectangle.set_scale_x_from_point(scale_x=ap.Number(1.5), x=x)
        >>> rectangle.get_scale_x_from_point(x=x)
        Number(1.5)
        """
        import apysc as ap
        from apysc._display import scale_interface_helper
        from apysc._type.expression_string import ExpressionString

        self._initialize_scale_x_from_point_if_not_initialized()
        default_val: ap.Number = ap.Number(1.0)
        key_exp_str: ExpressionString = (
            scale_interface_helper.get_coordinate_key_for_expression(
                coordinate=int(x._value)
            )
        )
        scale_x: ap.Number = self._scale_x_from_point.get(
            key=key_exp_str, default=default_val
        )
        return scale_x

    @final
    @arg_validation_decos.is_apysc_num(arg_position_index=1)
    @arg_validation_decos.is_apysc_integer(arg_position_index=2)
    @add_debug_info_setting(module_name=__name__)
    def set_scale_x_from_point(self, *, scale_x: Number, x: Int) -> None:
        """
        Update a scale-x value from the given x-coordinate.

        Parameters
        ----------
        scale_x : Number
            Scale-x value to set.
        x : Int
            X-coordinate.

        Notes
        -----
        The scale's minimum value is almost zero, and it does not become negative.

        References
        ----------
        - GraphicsBase scale_from_point interfaces
            - https://simon-ritchie.github.io/apysc/en/graphics_base_scale_from_point.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color="#0af")
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> x: ap.Int = ap.Int(100)
        >>> rectangle.set_scale_x_from_point(scale_x=ap.Number(1.5), x=x)
        >>> rectangle.get_scale_x_from_point(x=x)
        Number(1.5)
        """
        from apysc._display import scale_interface_helper
        from apysc._type.expression_string import ExpressionString

        self._initialize_scale_x_from_point_if_not_initialized()
        key_exp_str: ExpressionString = (
            scale_interface_helper.get_coordinate_key_for_expression(
                coordinate=int(x._value)
            )
        )
        self._set_lower_scale_limit(value=scale_x)
        self._scale_x_from_point._value[key_exp_str.value] = scale_x
        self._append_scale_x_from_point_update_expression(x=x)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_scale_x_from_point_update_expression(self, *, x: Int) -> None:
        """
        Append the scale-x from the specified x-coordinate updating
        expression.

        Parameters
        ----------
        x : Int
            X-coordinate.
        """
        import apysc as ap
        from apysc._display import scale_interface_helper

        expression: str
        expression = scale_interface_helper.get_scale_updating_expression(
            coordinate=x,
            scale_dict=self._scale_x_from_point,
            interface_variable_name=self.variable_name,
            coordinate_type=scale_interface_helper.CoordinateType.X,
        )
        ap.append_js_expression(expression=expression)

    _scale_x_from_point_snapshots: Dict[str, Dict[str, Any]]

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make a value snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_scale_x_from_point_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name="_scale_x_from_point_snapshots",
            value={**self._scale_x_from_point._value},
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert a value if a snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        if not self._snapshot_exists(snapshot_name=snapshot_name):
            return
        self._scale_x_from_point._value = self._scale_x_from_point_snapshots[
            snapshot_name
        ]



"""This module is for the translation mapping data of the
following document:

Document file: datetime_month.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# DateTime class month property": "# DateTime クラスの month 属性",
    ##################################################
    "This page explains the `DateTime` class's `month` property interface.": "このページでは`DateTime`クラスの`month`属性のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `month` property gets or sets a month's value.": "`month`属性では月の値の取得もしくは設定を行うことができます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "A `DateTime` instance has its property interface.": "`DateTime`クラスのインスタンスがその属性のインターフェイスを持っています。",  # noqa
    ##################################################
    "Its getter interface returns a month's `Int` value.": "そのgetterのインターフェイスは`Int`型の月の値を返却します。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\ndatetime_: ap.DateTime = ap.DateTime(year=2022, month=12, day=1)\nassert datetime_.month == 12\n```": "```py\n# runnable\nimport apysc as ap\n\ndatetime_: ap.DateTime = ap.DateTime(year=2022, month=12, day=1)\nassert datetime_.month == 12\n```",  # noqa
    ##################################################
    "Also, its setter interface accepts a month's `Int` value.": "また、setter側のインターフェイスでは同様に`Int`型の月の値を受け付けます。",  # noqa
    ##################################################
    "1-12 integer is acceptable.": "1～12の範囲の整数を受け付けることができます。",
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\ndatetime_: ap.DateTime = ap.DateTime(year=2022, month=12, day=1)\ndatetime_.month = ap.Int(1)\nassert datetime_.month == 1\n```": "```py\n# runnable\nimport apysc as ap\n\ndatetime_: ap.DateTime = ap.DateTime(year=2022, month=12, day=1)\ndatetime_.month = ap.Int(1)\nassert datetime_.month == 1\n```",  # noqa
    ##################################################
    "## month property API": "## month 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a current month's value.<hr>": "現在の月の値を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `month`: Int": "- `month`: Int",
    ##################################################
    "  - A current month value.": "  - 現在の月の値。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> datetime_: ap.DateTime = ap.DateTime(year=2022, month=12, day=1)\n>>> datetime_.month\nInt(12)\n\n>>> datetime_.month = ap.Int(1)\n>>> datetime_.month\nInt(1)\n```": "```py\n>>> import apysc as ap\n>>> datetime_: ap.DateTime = ap.DateTime(year=2022, month=12, day=1)\n>>> datetime_.month\nInt(12)\n\n>>> datetime_.month = ap.Int(1)\n>>> datetime_.month\nInt(1)\n```",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: datetime_now.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# DateTime class now interface": "# DateTime クラスの now インターフェイス",
    ##################################################
    "This page explains the `DateTime` class `now` method interface.": "このページでは`DateTime`クラスの`now`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `DateTime`'s `now` class method returns a current time `DateTime` instance.": "`DateTime`クラスの`now`のクラスメソッドは現在の時間での`DateTime`のインスタンスを返却します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `DateTime` class has the `now` method defined as a class method.": "`DateTime`クラスはクラスメソッドとして定義された`now`メソッドを持っています。",  # noqa
    ##################################################
    "It returns a `DateTime` instance, and its values become a current time.": "そのインターフェイスでは各値に現在時刻が設定された`DateTime`クラスのインスタンスを返却します。",  # noqa
    ##################################################
    "In the following example, if you click the rectangle, the click handler displays a current year, month, and day on the browser console.": "以下のコード例では、四角をクリックするとブラウザのコンソール上に現在の年月日を表示します。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent, options: dict) -> None:\n    """\n    The handler to handle a rectangle\'s click event.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        An event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    now_datetime: ap.DateTime = ap.DateTime.now()\n    ap.trace("Current year:", now_datetime.year)\n    ap.trace("Current month:", now_datetime.month)\n    ap.trace("Current day:", now_datetime.day)\n\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\nrectangle: ap.Rectangle = ap.Rectangle(\n    x=50, y=50, width=50, height=50, fill_color="#0af"\n)\nrectangle.click(on_click)\n\nap.save_overall_html(dest_dir_path="datetime_now_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent, options: dict) -> None:\n    """\n    The handler to handle a rectangle\'s click event.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        An event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    now_datetime: ap.DateTime = ap.DateTime.now()\n    ap.trace("Current year:", now_datetime.year)\n    ap.trace("Current month:", now_datetime.month)\n    ap.trace("Current day:", now_datetime.day)\n\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\nrectangle: ap.Rectangle = ap.Rectangle(\n    x=50, y=50, width=50, height=50, fill_color="#0af"\n)\nrectangle.click(on_click)\n\nap.save_overall_html(dest_dir_path="datetime_now_basic_usage/")\n```',  # noqa
    ##################################################
    "## now class method API": "## now のクラスメソッドのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a `DateTime` instance of the current time.<hr>": "現在時刻が設定された`DateTime`クラスのインスタンスを取得します。<hr>",  # noqa
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `dt`: DateTime": "- `dt`: DateTime",
    ##################################################
    "  - A created `DateTime` instance.": "  - 生成された`DateTime`クラスのインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> from datetime import datetime\n>>> import apysc as ap\n>>> py_now: datetime = datetime.now()\n>>> ap_now: ap.DateTime = ap.DateTime.now()\n>>> ap_now.year == py_now.year\nBoolean(True)\n\n>>> ap_now.month == py_now.month\nBoolean(True)\n\n>>> ap_now.day == py_now.day\nBoolean(True)\n```": "```py\n>>> from datetime import datetime\n>>> import apysc as ap\n>>> py_now: datetime = datetime.now()\n>>> ap_now: ap.DateTime = ap.DateTime.now()\n>>> ap_now.year == py_now.year\nBoolean(True)\n\n>>> ap_now.month == py_now.month\nBoolean(True)\n\n>>> ap_now.day == py_now.day\nBoolean(True)\n```",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: graphics_base_rotation_around_center.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# GraphicsBase rotation_around_center interface": "# GraphicsBase クラスの rotation_around_center インターフェイス",  # noqa
    ##################################################
    "This page explains the `GraphicsBase` class (base class of each graphic, such as the `Rectangle`) `rotation_around_center` property interface.": "このページでは`GraphicsBase`クラス（`Rectangle`などのグラフィックのクラスの基底クラス）の`rotation_around_center`属性のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `rotation_around_center` property interface can set the rotation angle to its instance (rotation value around its center point).": "`rotation_around_center`属性のインターフェイスではインスタンスの中央座標を基準とした回転角度の設定を行うことができます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `rotation_around_center` interface accepts the `int` or `Int` value.": "`rotation_around_center`インターフェイスは`int`もしくは`Int`の値を受け付けます。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\n\n# Set the cyan fill color and draw the rectangle.\nsprite.graphics.begin_fill(color="#0af", alpha=0.5)\ncyan_rect: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\ncyan_rect.rotation_around_center = ap.Int(30)\n\n# Set the magenta fill color and draw the rectangle.\nsprite.graphics.begin_fill(color="#f0a", alpha=0.5)\nmagenta_rect: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n# Append the rotation angle with the incremental addition (the result\n# rotation will be 60 degrees).\nmagenta_rect.rotation_around_center += ap.Int(30)\nmagenta_rect.rotation_around_center += ap.Int(30)\n\nap.save_overall_html(dest_dir_path="graphics_base_rotation_around_center_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\n\n# Set the cyan fill color and draw the rectangle.\nsprite.graphics.begin_fill(color="#0af", alpha=0.5)\ncyan_rect: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\ncyan_rect.rotation_around_center = ap.Int(30)\n\n# Set the magenta fill color and draw the rectangle.\nsprite.graphics.begin_fill(color="#f0a", alpha=0.5)\nmagenta_rect: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n# Append the rotation angle with the incremental addition (the result\n# rotation will be 60 degrees).\nmagenta_rect.rotation_around_center += ap.Int(30)\nmagenta_rect.rotation_around_center += ap.Int(30)\n\nap.save_overall_html(dest_dir_path="graphics_base_rotation_around_center_basic_usage/")\n```',  # noqa
    ##################################################
    "## Notes": "## 特記事項",
    ##################################################
    "This interface supports only the graphics instances currently. The container instances, such as the `Sprite` instance, are not supported (due to the HTML (SVG) specification).": "このインターフェイスは現在グラフィック系のクラスでのみサポートしており、`Sprite`などのコンテナーのインスタンスでは現在サポートしていません（HTMLのSVGの仕様に依存しています）。",  # noqa
    ##################################################
    "## rotation_around_center property API": "## rotation_around_center 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a rotation value around the center of this instance.<hr>": "インスタンスの中央座標を基準とした回転量を取得します。<hr>",  # noqa
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `rotation_around_center`: Int": "- `rotation_around_center`: Int",
    ##################################################
    "  - Rotation value around the center of this instance.": "  - このインスタンスの中央座標を基準とした回転量。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> rectangle.rotation_around_center = ap.Int(45)\n>>> rectangle.rotation_around_center\nInt(45)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> rectangle.rotation_around_center = ap.Int(45)\n>>> rectangle.rotation_around_center\nInt(45)\n```',  # noqa
}



"""Graphics class-related expression implementations.
"""

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.string import String


@add_debug_info_setting(module_name=__name__)
def append_fill_expression(
    *, fill_color: String, expression: str, indent_num: int
) -> str:
    """
    Append a fill expression to specified expression's string.

    Parameters
    ----------
    fill_color : String
        A fill-color to use.
    expression : str
        Expression string to be appended fill expression.
    indent_num : int
        Indentation number.

    Returns
    -------
    expression : str
        After appended expression string.
    """
    from apysc._string import indent_util

    spaces: str = indent_util.make_spaces_for_html(indent_num=indent_num)
    if fill_color._value == "":
        expression += f'\n{spaces}fill: "none",'
        return expression
    expression += f"\n{spaces}fill: {fill_color.variable_name},"
    return expression


@add_debug_info_setting(module_name=__name__)
def append_fill_opacity_expression(
    *, fill_alpha: Number, expression: str, indent_num: int
) -> str:
    """
    Append a fill opacity expression to a specified expression's string.

    Parameters
    ----------
    fill_alpha : Number
        A fill-alpha to use.
    expression : str
        Expression string to be appended fill opacity expression.
    indent_num : int
        Indentation number.

    Returns
    -------
    expression : str
        After appended expression string.
    """
    from apysc._string import indent_util

    spaces: str = indent_util.make_spaces_for_html(indent_num=indent_num)
    expression += f'\n{spaces}"fill-opacity": {fill_alpha.variable_name},'
    return expression


@add_debug_info_setting(module_name=__name__)
def append_x_expression(*, x: Int, expression: str, indent_num: int) -> str:
    """
    Append x position expression to specified expression's string.

    Parameters
    ----------
    x : Int
        An x-coordinate to use.
    expression : str
        Expression string to be appended x position expression.
    indent_num : int
        Indentation number.

    Returns
    -------
    expression : str
        After appended expression string.
    """
    from apysc._string import indent_util

    spaces: str = indent_util.make_spaces_for_html(indent_num=indent_num)
    expression += f"\n{spaces}x: {x.variable_name},"
    return expression


@add_debug_info_setting(module_name=__name__)
def append_y_expression(*, y: Int, expression: str, indent_num: int) -> str:
    """
    Append y position expression to specified expression's string.

    Parameters
    ----------
    y : Int
        A y-coordinate to use.
    expression : str
        Expression string to be appended y position expression.
    indent_num : int
        Indentation number.

    Returns
    -------
    expression : str
        After appended expression string.
    """
    from apysc._string import indent_util

    spaces: str = indent_util.make_spaces_for_html(indent_num=indent_num)
    expression += f"\n{spaces}y: {y.variable_name},"
    return expression


@add_debug_info_setting(module_name=__name__)
def append_stroke_expression(
    *, line_color: String, expression: str, indent_num: int
) -> str:
    """
    Append stroke expression to specified expression's string.

    Parameters
    ----------
    line_color : String
        A line-color to use.
    expression : str
        Expression string to be appended stroke expression.
    indent_num : int
        Indentation number.

    Returns
    -------
    expression : str
        After appended expression string.
    """
    from apysc._string import indent_util

    if line_color._value == "":
        return expression
    spaces: str = indent_util.make_spaces_for_html(indent_num=indent_num)
    expression += f"\n{spaces}stroke: {line_color.variable_name},"
    return expression


@add_debug_info_setting(module_name=__name__)
def append_stroke_width_expression(
    *, line_thickness: Int, expression: str, indent_num: int
) -> str:
    """
    Append stroke width expression to specified expression's string.

    Parameters
    ----------
    line_thickness : Int
        A line-thickness to use.
    expression : str
        Expression string to be appended stroke width expression.
    indent_num : int
        Indentation number.

    Returns
    -------
    expression : str
        After appended expression string.
    """
    from apysc._string import indent_util

    spaces: str = indent_util.make_spaces_for_html(indent_num=indent_num)
    expression += f'\n{spaces}"stroke-width": ' f"{line_thickness.variable_name},"
    return expression


@add_debug_info_setting(module_name=__name__)
def append_stroke_opacity_expression(
    *, line_alpha: Number, expression: str, indent_num: int
) -> str:
    """
    Append stroke opacity expression to specified expression's string.

    Parameters
    ----------
    line_alpha : Number
        A line-alpha to use.
    expression : str
        Expression string to be appended stroke opacity expression.
    indent_num : int
        Indentation number.

    Returns
    -------
    expression : str
        After appended expression string.
    """
    from apysc._string import indent_util

    spaces: str = indent_util.make_spaces_for_html(indent_num=indent_num)
    expression += f'\n{spaces}"stroke-opacity": {line_alpha.variable_name},'
    return expression


@add_debug_info_setting(module_name=__name__)
def append_stroke_linecap_expression(
    *, line_cap: String, expression: str, indent_num: int
) -> str:
    """
    Append stroke line cap expression to specified expression's string.

    Parameters
    ----------
    line_cap : String
        A line-cap to use.
    expression : str
        Expression string to be appended stroke linecap expression.
    indent_num : int
        Indentation number.

    Returns
    -------
    expression : str
        After appended expression string.
    """
    import apysc as ap
    from apysc._string import indent_util

    if line_cap._value == ap.LineCaps.BUTT.value:
        return expression
    spaces: str = indent_util.make_spaces_for_html(indent_num=indent_num)
    expression += f'\n{spaces}"stroke-linecap": {line_cap.variable_name},'
    return expression


@add_debug_info_setting(module_name=__name__)
def append_stroke_linejoin_expression(
    *, line_joints: String, expression: str, indent_num: int
) -> str:
    """
    Append stroke line-join expression to specified
    expression's string.

    Parameters
    ----------
    line_joints : String
        A line-join to use.
    expression : str
        Expression string to be appended stroke linejoin expression.
    indent_num : int
        Indentation number.

    Returns
    -------
    expression : str
        After appended expression string.
    """
    import apysc as ap
    from apysc._string import indent_util

    if line_joints._value == ap.LineJoints.MITER.value:
        return expression
    spaces: str = indent_util.make_spaces_for_html(indent_num=indent_num)
    expression += f'\n{spaces}"stroke-linejoin": ' f"{line_joints.variable_name},"
    return expression



"""Color-related implementations.
"""

from typing import Any
from typing import TypeVar

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.string import String

StrOrString = TypeVar("StrOrString", str, String)


@add_debug_info_setting(module_name=__name__)
def complement_hex_color(*, hex_color_code: StrOrString) -> StrOrString:
    """
    Complement hex color for convenience; for instance,
    add # prefix or three digits to become six digits,
    upper case to lower case.

    Parameters
    ----------
    hex_color_code : str or String
        Hexadecimal color code. e.g., 'ff0000', '#666', '#0'

    Returns
    -------
    complemented_hex_color_code : str or String
        Result hex color code. e.g., '#ff0000', '#666666, '#000000'
    """
    import apysc as ap
    from apysc._validation import color_validation

    hex_color_code = remove_color_code_sharp_symbol(hex_color_code=hex_color_code)

    color_validation.validate_hex_color_code_format(hex_color_code=hex_color_code)
    if isinstance(hex_color_code, ap.String):
        value_: str = hex_color_code._value
    else:
        value_ = str(hex_color_code)
    char_len = len(value_)
    if char_len == 1:
        value_ = _fill_one_digit_hex_color_code(hex_color_code=value_)
    elif char_len == 3:
        value_ = _fill_three_digit_hex_color_code(hex_color_code=value_)
    value_ = value_.lower()
    value_ = f"#{value_}"

    if isinstance(hex_color_code, ap.String):
        hex_color_code._value = value_
        _append_complement_hex_color_expression(hex_color_code=hex_color_code)
    else:
        hex_color_code = value_  # type: ignore
    return hex_color_code  # type: ignore


def remove_color_code_sharp_symbol(*, hex_color_code: StrOrString) -> StrOrString:
    """
    Remove a sharp symbol from a specified hexadecimal color code
    string.

    Parameters
    ----------
    hex_color_code : StrOrString
        A targert hexadecimal color code string.

    Returns
    -------
    hex_color_code : StrOrString
        A result string.
    """
    from apysc._type import value_util

    if isinstance(hex_color_code, str):
        hex_color_code_: str = hex_color_code  # type: ignore
        if hex_color_code_.startswith("#"):
            hex_color_code_ = hex_color_code_[1:]
        return hex_color_code_  # type: ignore

    if isinstance(hex_color_code, String):
        hex_color_code__: String = value_util.get_copy(value=hex_color_code)
        if hex_color_code__._value.startswith("#"):
            hex_color_code__._value = hex_color_code__._value[1:]
        _append_remove_color_code_sharp_symbol_expression(
            hex_color_code=hex_color_code__
        )
        return hex_color_code__  # type: ignore

    raise TypeError(
        "Other than str or String type value is specified: " f"{type(hex_color_code)}"
    )


def _append_remove_color_code_sharp_symbol_expression(
    *, hex_color_code: String
) -> None:
    """
    Append the remove_color_code_sharp_symbol function's
    expression.

    Parameters
    ----------
    hex_color_code : String
        A sharp symbol removed string instance.
    """
    import apysc as ap

    var_name: str = hex_color_code.variable_name
    expression: str = (
        f"var first_char = {var_name}.slice(0, 1);"
        '\nif (first_char === "#") {'
        f"\n  {var_name} = {var_name}.slice(1);"
        "\n}"
    )
    ap.append_js_expression(expression=expression)


@add_debug_info_setting(module_name=__name__)
def _append_complement_hex_color_expression(*, hex_color_code: Any) -> None:
    """
    Append complement_hex_color function's expression.

    Parameters
    ----------
    hex_color_code : String
        Complemented hex color code string.
    """
    import apysc as ap
    from apysc._expression import expression_variables_util
    from apysc._expression import var_names

    index_name: str = expression_variables_util.get_next_variable_name(
        type_name=var_names.INDEX
    )
    hex_color_code_: ap.String = hex_color_code
    var_name: str = hex_color_code_.variable_name
    expression: str = (
        f"var str_length = {var_name}.length;"
        "\nif (str_length === 1) {"
        f'\n  {var_name} = "00000" + {var_name};'
        "\n}else if (str_length === 3) {"
        f'\n  var {var_name}_ = "";'
        f"\n  for (var {index_name} = 0; {index_name} < "
        f"{var_name}.length; "
        f"{index_name}++) {{"
        f"\n    {var_name}_ += {var_name}[{index_name}].repeat(2);"
        "\n  }"
        f"\n  {var_name} = {var_name}_;"
        "\n}"
        f'\n{var_name} = "#" + {var_name};'
    )
    ap.append_js_expression(expression=expression)


def _fill_three_digit_hex_color_code(*, hex_color_code: str) -> str:
    """
    Fill 3 digits hexadecimal color code until it becomes 6 digits.

    Parameters
    ----------
    hex_color_code : str
        One digit hexadecimal color code (not including '#').
        e.g., 'aaa', 'fff'

    Returns
    -------
    filled_color_code : str
        Result color code. e.g., 'aaaaaa', 'ffffff'
    """
    filled_color_code: str = ""
    for char in hex_color_code:
        filled_color_code += char * 2
    return filled_color_code


def _fill_one_digit_hex_color_code(*, hex_color_code: str) -> str:
    """
    Fill 1 digit hexadecimal color code until it becomes 6 digits.

    Parameters
    ----------
    hex_color_code : str
        One digit hexadecimal color code (not including '#').
        e.g., 'a', '0'

    Returns
    -------
    filled_color_code : str
        Result color code. e.g., '00000a', '000000'
    """
    filled_color_code: str = hex_color_code.zfill(6)
    return filled_color_code



"""Base class implementation for the path data.
"""

from abc import ABC
from abc import abstractmethod
from typing import Union

from typing_extensions import final

from apysc._branch._else import Else
from apysc._branch._if import If
from apysc._geom.path_label import PathLabel
from apysc._geom.relative_mixin import RelativeMixIn
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.attr_to_apysc_val_from_builtin_mixin import (
    AttrToApyscValFromBuiltinMixIn,
)
from apysc._type.boolean import Boolean
from apysc._type.string import String
from apysc._validation import arg_validation_decos


class PathDataBase(RelativeMixIn, AttrToApyscValFromBuiltinMixIn, ABC):
    """
    Base class for the path data.
    """

    _path_label: PathLabel

    @arg_validation_decos.is_boolean(arg_position_index=2)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self, *, path_label: PathLabel, relative: Union[bool, Boolean]
    ) -> None:
        """
        Base class for the path data.

        Parameters
        ----------
        path_label : PathLabel
            Target (SVG's) path label.
        relative : bool or Boolean
            A boolean value indicates whether the path
            coordinates are relative or not (absolute).
        """
        self._path_label = path_label
        self.relative = self._get_copied_boolean_from_builtin_val(
            bool_val=relative, attr_identifier="relative"
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def _get_svg_char(self) -> String:
        """
        Get a SVG character (e.g., 'M' or 'm') from the
        current setting.

        Returns
        -------
        svg_char : String
            Target SVG character.
        """
        svg_char_: str = self._path_label.value
        if self._relative._value:
            svg_char: String = String(
                svg_char_.lower(), variable_name_suffix=self._variable_name_suffix
            )
        else:
            svg_char = String(
                svg_char_, variable_name_suffix=self._variable_name_suffix
            )
        with If(self._relative, locals_=locals()):
            svg_char.value = svg_char_.lower()
        with Else(locals_=locals()):
            svg_char.value = svg_char_
        return svg_char

    @abstractmethod
    def _get_svg_str(self) -> str:
        """
        Get a path's SVG string created with the current setting.
        """



"""This module is for the translation mapping data of the
following document:

Document file: timer_event.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# TimerEvent class": "# TimerEvent クラス",
    ##################################################
    "This page explains the `TimerEvent` class.": "このページでは`TimerEvent`クラスについて説明します。",
    ##################################################
    "## What class is this?": "## クラス概要",
    ##################################################
    "The `TimerEvent` class is the event class that a timer passes to a timer event handler function, such as the `Timer` class constructor or the `timer_complete` function\\.": "`TimerEvent`クラスは`Timer`クラスのコンストラクタや`timer_complete`などのインターフェイスで登録されるタイマー関係のイベントのハンドラに渡されるイベントのクラスです。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "Each timer event handler's first argument becomes the `TimerEvent` class instance.": "タイマー関係の各イベントハンドラの第一引数は`TimerEvent`クラスのインスタンスとなります。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.rotation_around_center += 1\n\n\ndef on_timer_complete(e: ap.TimerEvent, options: dict) -> None:\n    """\n    The handler that the timer calls when completed.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.trace("Timer complete!")\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\noptions: _RectOptions = {"rectangle": rectangle}\ntimer: ap.Timer = ap.Timer(handler=on_timer, delay=33.3, options=options)\ntimer.start()\ntimer.timer_complete(handler=on_timer_complete)\n\nap.save_overall_html(dest_dir_path="timer_event_basic_usage/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.rotation_around_center += 1\n\n\ndef on_timer_complete(e: ap.TimerEvent, options: dict) -> None:\n    """\n    The handler that the timer calls when completed.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.trace("Timer complete!")\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\noptions: _RectOptions = {"rectangle": rectangle}\ntimer: ap.Timer = ap.Timer(handler=on_timer, delay=33.3, options=options)\ntimer.start()\ntimer.timer_complete(handler=on_timer_complete)\n\nap.save_overall_html(dest_dir_path="timer_event_basic_usage/")\n```',  # noqa
    ##################################################
    "## this attribute": "## this 属性",
    ##################################################
    "The `TimerEvent` instance's `this` attribute becomes the target `Timer` instance, and you can use each timer instance interface from it.": "`TimerEvent`のインスタンスの`this`属性は対象の`Timer`クラスのインスタンスとなり、ぞれを参照してタイマー関係の各インターフェイスを使用することができます。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.rotation_around_center += 1\n    ap.trace("Current timer count: ", e.this.current_count)\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\noptions: _RectOptions = {"rectangle": rectangle}\ntimer: ap.Timer = ap.Timer(handler=on_timer, delay=16.6, options=options)\ntimer.start()\n\nap.save_overall_html(dest_dir_path="timer_event_this_attribute/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.rotation_around_center += 1\n    ap.trace("Current timer count: ", e.this.current_count)\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\noptions: _RectOptions = {"rectangle": rectangle}\ntimer: ap.Timer = ap.Timer(handler=on_timer, delay=16.6, options=options)\ntimer.start()\n\nap.save_overall_html(dest_dir_path="timer_event_this_attribute/")\n```',  # noqa
    ##################################################
    "## TimerEvent constructor API": "## TimerEvent クラスのコンストラクタのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Timer event class.<hr>": "タイマー関係のイベントのクラスです。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `this`: Timer": "- `this`: Timer",
    ##################################################
    "  - Target timer instance.": "  - 対象のタイマーのインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.x += 1\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> ap.Timer(\n...     on_timer,\n...     delay=ap.FPS.FPS_60,\n...     options=options,\n... ).start()\n```': '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.x += 1\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> ap.Timer(\n...     on_timer,\n...     delay=ap.FPS.FPS_60,\n...     options=options,\n... ).start()\n```',  # noqa
    ##################################################
    "## this attribute API": "## this 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a timer instance of listening to this event.<hr>": "このイベントのハンドラが設定されている対象のタイマーのインスタンス。<hr>",  # noqa
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `this`: TImer": "- `this`: TImer",
    ##################################################
    "  - Instance of listening to this event.": "  - このイベントのハンドラが設定されているインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.x += 1\n...     with ap.If(rectangle.x >= 100):\n...         timer: ap.Timer = e.this\n...         timer.stop()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> ap.Timer(\n...     on_timer,\n...     delay=ap.FPS.FPS_60,\n...     options=options,\n... ).start()\n```': '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.x += 1\n...     with ap.If(rectangle.x >= 100):\n...         timer: ap.Timer = e.this\n...         timer.stop()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> ap.Timer(\n...     on_timer,\n...     delay=ap.FPS.FPS_60,\n...     options=options,\n... ).start()\n```',  # noqa
}



"""This module is for the definitions of the mouse
event types.
"""

from enum import Enum


class MouseEventType(Enum):
    """
    Mouse event type definitions.
    """

    CLICK = "click"
    DBLCLICK = "dblclick"
    MOUSEDOWN = "mousedown"
    MOUSEUP = "mouseup"
    MOUSEOVER = "mouseover"
    MOUSEOUT = "mouseout"
    MOUSEMOVE = "mousemove"



"""This module is for the translation mapping data of the
following document:

Document file: point2d.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Point2D class": "# Point2D クラス",
    ##################################################
    "This page explains the `Point2D` class.": "このページでは`Point2D`クラスについて説明します。",
    ##################################################
    "## What is the Point2D class?": "## Point2D クラスの概要",
    ##################################################
    "The `Point2D` class is the 2D coordinates class interface. This interface handles the x-coordinate and y-coordinate. This interface is used, for example, the `Polygon` class drawing to specify each vertex point.": "`Point2D`クラスは2次元の座標値を扱うためのインターフェイスのクラスで、X座標とY座標を扱います。このクラスは`Polygon`クラスなどの一部のクラスで使用され、各頂点座標の設定用に参照されます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `Point2D` class constructor requires the `x` and `y` arguments. Both parameters type is the Python built-in `int` or `Int`\\.": "`Point2D`クラスのコンストラクタでは`x`と`y`の各引数が必要になります。各引数はPythonのビルトインの`int`の値かもしくはapyscの`Int`の値を受け付けます。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\npoint_1: ap.Point2D = ap.Point2D(x=10, y=20)\n\nx: ap.Int = ap.Int(10)\ny: ap.Int = ap.Int(20)\npoint_2: ap.Point2D = ap.Point2D(x=x, y=y)\n```": "```py\n# runnable\nimport apysc as ap\n\npoint_1: ap.Point2D = ap.Point2D(x=10, y=20)\n\nx: ap.Int = ap.Int(10)\ny: ap.Int = ap.Int(20)\npoint_2: ap.Point2D = ap.Point2D(x=x, y=y)\n```",  # noqa
    ##################################################
    "## X and y getter interfaces": "## XとY座標のgetterのインターフェイス",
    ##################################################
    "The `Point2D` class `x` and `y` property interfaces returns the `Int` type value, as follows:": "`Point2D`クラスの`x`と`y`の属性は以下のコード例のように`Int`型の値を返却します:",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\npoint: ap.Point2D = ap.Point2D(x=10, y=20)\nassert point.x == 10\nassert point.y == 20\n```": "```py\n# runnable\nimport apysc as ap\n\npoint: ap.Point2D = ap.Point2D(x=10, y=20)\nassert point.x == 10\nassert point.y == 20\n```",  # noqa
    ##################################################
    "## X and y setter interfaces": "## XとY座標のsetterのインターフェイス",
    ##################################################
    "The `x` and `y` property can be updated with an `Int` type value, as follows:": "`x`と`y`属性は以下のコード例のように`Int`型の値を使って値を更新することができます:",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\npoint: ap.Point2D = ap.Point2D(x=10, y=20)\npoint.x = ap.Int(30)\nassert point.x == 30\n```": "```py\n# runnable\nimport apysc as ap\n\npoint: ap.Point2D = ap.Point2D(x=10, y=20)\npoint.x = ap.Int(30)\nassert point.x == 30\n```",  # noqa
    ##################################################
    "## Usage example of the draw_polygon interface": "## draw_polygon インターフェイスにおける使用例",  # noqa
    ##################################################
    "The `draw_polygon` interface requires the `Point2D` list argument so that this section shows the example of the `Point2D` class with that drawing interface.": "`draw_polygon`インターフェイスは`Point2D`の値のリストの引数を必要とします。そのためこの節ではその描画のインターフェイスによる`Point2D`クラスを使ったコード例を載せています。",  # noqa
    ##################################################
    "The following draws the triangle vector graphics by specifying the three points:": "以下のコード例では3点の座標を指定することによって三角形のベクターグラフィックスを描画しています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\nsprite.graphics.draw_polygon(\n    points=[\n        ap.Point2D(x=75, y=50),\n        ap.Point2D(x=50, y=100),\n        ap.Point2D(x=100, y=100),\n    ]\n)\n\nap.save_overall_html(dest_dir_path="point2d_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\nsprite.graphics.draw_polygon(\n    points=[\n        ap.Point2D(x=75, y=50),\n        ap.Point2D(x=50, y=100),\n        ap.Point2D(x=100, y=100),\n    ]\n)\n\nap.save_overall_html(dest_dir_path="point2d_basic_usage/")\n```',  # noqa
    ##################################################
    "## Point2D class constructor API": "## Point2D クラスのコンストラクタのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "2-dimensional geometry point.<hr>": "2次元の座標値を扱うクラスです。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `x`: int or Int": "- `x`: int or Int",
    ##################################################
    "  - X-coordinate.": "  - X座標。",
    ##################################################
    "- `y`: int or Int": "- `y`: int or Int",
    ##################################################
    "  - Y-coordinate.": "  - Y座標。",
    ##################################################
    "- `variable_name_suffix`: str, default ''": "- `variable_name_suffix`: str, default ''",  # noqa
    ##################################################
    "  - A JavaScript variable name suffix string. This setting is sometimes useful for JavaScript's debugging.": "  - JavaScript上の変数のサフィックスの設定です。この設定はJavaScriptのデバッグ時に役立つことがあります。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> polygon: ap.Polygon = sprite.graphics.draw_polygon(\n...     points=[\n...         ap.Point2D(x=0, y=0),\n...         ap.Point2D(x=0, y=50),\n...         ap.Point2D(x=50, y=25),\n...     ]\n... )\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> polygon: ap.Polygon = sprite.graphics.draw_polygon(\n...     points=[\n...         ap.Point2D(x=0, y=0),\n...         ap.Point2D(x=0, y=50),\n...         ap.Point2D(x=50, y=25),\n...     ]\n... )\n```',  # noqa
    ##################################################
    "## x property API": "## x属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "X-coordinate property.<hr>": "X座標の属性のインターフェイスです。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `x`: Int": "- `x`: Int",
    ##################################################
    "  - X-coordinate.": "  - X座標。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> point: ap.Point2D = ap.Point2D(x=50, y=100)\n>>> point.x = ap.Int(150)\n>>> point.x\nInt(150)\n```": "```py\n>>> import apysc as ap\n>>> point: ap.Point2D = ap.Point2D(x=50, y=100)\n>>> point.x = ap.Int(150)\n>>> point.x\nInt(150)\n```",  # noqa
    ##################################################
    "## y property API": "## y属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Y-coordinate property.<hr>": "Y座標の属性のインターフェイスです。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `y`: Int": "- `y`: Int",
    ##################################################
    "  - Y-coordinate.": "  - Y座標。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> point: ap.Point2D = ap.Point2D(x=50, y=100)\n>>> point.y = ap.Int(150)\n>>> point.y\nInt(150)\n```": "```py\n>>> import apysc as ap\n>>> point: ap.Point2D = ap.Point2D(x=50, y=100)\n>>> point.y = ap.Int(150)\n>>> point.y\nInt(150)\n```",  # noqa
}



"""This module is for the variable name's suffix setting mix-in
class.
"""


class VariableNameSuffixMixIn:

    __variable_name_suffix: str = ""

    @property
    def _variable_name_suffix(self) -> str:
        """
        Get a variable name suffix string.

        Returns
        -------
        variable_name_suffix : str
            A variable name suffix string.
        """
        return self.__variable_name_suffix

    @_variable_name_suffix.setter
    def _variable_name_suffix(self, value: str) -> None:
        """
        Set a variable name suffix string.

        Parameters
        ----------
        value : str
            A string to set.
        """
        self.__variable_name_suffix = value



"""This module is for the translation mapping data of the
following document:

Document file: array_append_and_push.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Array class append and push interfaces": "# Array クラスの append と push のインターフェイス",
    ##################################################
    "This page explains the `Array` class `append` and `push` method interfaces.": "このページでは`Array`クラスの`append`と`push`メソッドの各インターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interfaces are these?": "## 各インターフェイスの概要",
    ##################################################
    "The `append` and `push` method interfaces append any value to the end of an array. These interfaces behave the same (`append` is similar to the Python built-in and the `push` interface is similar to the JavaScript).": "`append`と`push`メソッドの各インターフェイスは配列の末端に任意の値を追加します。これらの各インターフェイスはお互いに同じ挙動をします（`append`はPython寄りな名前であり、`push`はJavaScript寄りな名前でエイリアスとして設けてあります）。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `append` and `push` methods require the first argument of the `value`\\.": "`append`と`push`の各メソッドは第一引数に`value`という引数名で追加する値の指定が必要になります。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 2])\narr.append(value=3)\nassert arr == [1, 2, 3]\n\narr.push(value=4)\nassert arr == [1, 2, 3, 4]\n```": "```py\n# runnable\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 2])\narr.append(value=3)\nassert arr == [1, 2, 3]\n\narr.push(value=4)\nassert arr == [1, 2, 3, 4]\n```",  # noqa
    ##################################################
    "## append API": "## append API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Add any value to the end of this array. This method behaves the same `push` method.<hr>": "任意の値をこの配列の末尾に加えます。このメソッドは`push`メソッドと同様に動作します。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `value`: *": "- `value`: *",
    ##################################################
    "  - Any value to append.": "  - 追加対象の任意の値。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> arr.append(4)\n>>> arr\nArray([1, 2, 3, 4])\n```": "```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> arr.append(4)\n>>> arr\nArray([1, 2, 3, 4])\n```",  # noqa
    ##################################################
    "## push API": "## push API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Add any value to the end of this array. This interface behaves the same as the `append` method.<hr>": "任意の値をこの配列の末尾に加えます。このメソッドは`append`メソッドと同様に動作します。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `value`: *": "- `value`: *",
    ##################################################
    "  - Any value to append.": "  - 追加対象の任意の値。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> arr.push(4)\n>>> arr\nArray([1, 2, 3, 4])\n```": "```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> arr.push(4)\n>>> arr\nArray([1, 2, 3, 4])\n```",  # noqa
}



"""isort:skip_file"""

# flake8: noqa

from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.number import Number as Float
from apysc._type.boolean import Boolean
from apysc._type.boolean import Boolean as Bool
from apysc._type.string import String
from apysc._type.array import Array
from apysc._type.dictionary import Dictionary
from apysc._type.any_value import AnyValue
from apysc._branch._if import If
from apysc._branch._elif import Elif
from apysc._branch._else import Else
from apysc._loop._for import For
from apysc._loop._continue import Continue
from apysc._display.display_object import DisplayObject
from apysc._display._document import document
from apysc._display.sprite import Sprite
from apysc._display.graphics import Graphics
from apysc._display.stage import Stage
from apysc._display.stage import get_stage
from apysc._display.rectangle import Rectangle
from apysc._display.circle import Circle
from apysc._display.ellipse import Ellipse
from apysc._display.line import Line
from apysc._display.polyline import Polyline
from apysc._display.polygon import Polygon
from apysc._display.line_caps import LineCaps
from apysc._display.line_joints import LineJoints
from apysc._display.line_dot_setting import LineDotSetting
from apysc._display.line_dash_setting import LineDashSetting
from apysc._display.line_round_dot_setting import LineRoundDotSetting
from apysc._display.line_dash_dot_setting import LineDashDotSetting
from apysc._display.path import Path
from apysc._geom.point2d import Point2D
from apysc._geom.path_label import PathLabel
from apysc._geom.path_data_base import PathDataBase
from apysc._geom.path_move_to import PathMoveTo
from apysc._geom.path_line_to import PathLineTo
from apysc._geom.path_horizontal import PathHorizontal
from apysc._geom.path_vertical import PathVertical
from apysc._geom.path_close import PathClose
from apysc._geom.path_bezier_2d import PathBezier2D
from apysc._geom.path_bezier_2d_continual import PathBezier2DContinual
from apysc._geom.path_bezier_3d import PathBezier3D
from apysc._geom.path_bezier_3d_continual import PathBezier3DContinual
from apysc._geom.path_data import PathData
from apysc._event.event import Event
from apysc._event.mouse_event import MouseEvent
from apysc._event.wheel_event import WheelEvent
from apysc._event.timer_event import TimerEvent
from apysc._event.animation_event import AnimationEvent
from apysc._event.enter_frame_event import EnterFrameEvent
from apysc._event.mouse_event_type import MouseEventType
from apysc._console._trace import trace
from apysc._console._trace import trace as print
from apysc._console.assertion import assert_equal
from apysc._console.assertion import assert_not_equal
from apysc._console.assertion import assert_true
from apysc._console.assertion import assert_false
from apysc._console.assertion import assert_greater
from apysc._console.assertion import assert_greater_equal
from apysc._console.assertion import assert_less
from apysc._console.assertion import assert_less_equal
from apysc._console.assertion import assert_arrays_equal
from apysc._console.assertion import assert_arrays_not_equal
from apysc._console.assertion import assert_dicts_equal
from apysc._console.assertion import assert_dicts_not_equal
from apysc._console.assertion import assert_defined
from apysc._console.assertion import assert_undefined
from apysc._event.document_mouse_wheel_func import bind_wheel_event_to_document
from apysc._event.document_mouse_wheel_func import unbind_wheel_event_all_from_document
from apysc._event.document_mouse_wheel_func import unbind_wheel_event_from_document
from apysc._html.exporter import save_overall_html
from apysc._expression.expression_data_util import append_js_expression
from apysc._jupyter.jupyter_util import display_on_jupyter
from apysc._jupyter.jupyter_util import display_on_colaboratory
from apysc._time.fps import FPS
from apysc._time.timer import Timer
from apysc._time.datetime_ import DateTime
from apysc._html.debug_mode import set_debug_mode
from apysc._html.debug_mode import unset_debug_mode
from apysc._html.debug_mode import is_debug_mode
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type._return import Return
from apysc._type._delete import delete
from apysc._animation.easing import Easing
from apysc._animation.animation_base import AnimationBase
from apysc._animation.animation_move import AnimationMove
from apysc._animation.animation_x import AnimationX
from apysc._animation.animation_y import AnimationY
from apysc._animation.animation_cx import AnimationCx
from apysc._animation.animation_cy import AnimationCy
from apysc._animation.animation_width import AnimationWidth
from apysc._animation.animation_height import AnimationHeight
from apysc._animation.animation_width_for_ellipse import AnimationWidthForEllipse
from apysc._animation.animation_height_for_ellipse import AnimationHeightForEllipse
from apysc._animation.animation_radius import AnimationRadius
from apysc._animation.animation_fill_alpha import AnimationFillAlpha
from apysc._animation.animation_fill_color import AnimationFillColor
from apysc._animation.animation_line_color import AnimationLineColor
from apysc._animation.animation_line_alpha import AnimationLineAlpha
from apysc._animation.animation_line_thickness import AnimationLineThickness
from apysc._animation.animation_rotation_around_center import AnimationRotationAroundCenter
from apysc._animation.animation_rotation_around_point import AnimationRotationAroundPoint
from apysc._animation.animation_scale_x_from_center import AnimationScaleXFromCenter
from apysc._animation.animation_scale_y_from_center import AnimationScaleYFromCenter
from apysc._animation.animation_scale_x_from_point import AnimationScaleXFromPoint
from apysc._animation.animation_scale_y_from_point import AnimationScaleYFromPoint
from apysc._animation.animation_parallel import AnimationParallel

__version__: str = "2.5.66"



"""Class implementation for line dash-dot setting mix-in.
"""

from typing import Dict
from typing import Optional

from typing_extensions import final

from apysc._display.line_dash_dot_setting import LineDashDotSetting
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._validation import arg_validation_decos


class LineDashDotSettingMixIn(VariableNameMixIn, RevertMixIn):

    _line_dash_dot_setting: Optional[LineDashDotSetting]

    @final
    def _initialize_line_dash_dot_setting_if_not_initialized(self) -> None:
        """
        Initialize _line_dash_dot_setting attribute if this interface does not
        initialize it yet.
        """
        if hasattr(self, "_line_dash_dot_setting"):
            return
        self._line_dash_dot_setting = None

    @property
    @add_debug_info_setting(module_name=__name__)
    def line_dash_dot_setting(self) -> Optional[LineDashDotSetting]:
        """
        Get current dash dot (1-dot chain) setting.

        Returns
        -------
        line_dash_dot_setting : LineDashDotSetting or None
            Dash dot (1-dot chain) setting.

        References
        ----------
        - GraphicsBase line_dash_dot_setting interface
            - https://simon-ritchie.github.io/apysc/en/graphics_base_line_dash_dot_setting.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color="#fff", thickness=10)
        >>> line: ap.Line = sprite.graphics.draw_line(
        ...     x_start=50, y_start=50, x_end=150, y_end=50
        ... )
        >>> line.line_dash_dot_setting = ap.LineDashDotSetting(
        ...     dot_size=2, dash_size=5, space_size=3
        ... )
        >>> line.line_dash_dot_setting.dot_size
        Int(2)

        >>> line.line_dash_dot_setting.dash_size
        Int(5)

        >>> line.line_dash_dot_setting.space_size
        Int(3)
        """
        self._initialize_line_dash_dot_setting_if_not_initialized()
        return self._line_dash_dot_setting

    @line_dash_dot_setting.setter
    @arg_validation_decos.multiple_line_settings_are_not_set(arg_position_index=0)
    @arg_validation_decos.is_line_dash_dot_setting(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def line_dash_dot_setting(self, value: Optional[LineDashDotSetting]) -> None:
        """
        Set line dash dot (1-dot chain) setting.

        Parameters
        ----------
        value : LineDashDotSetting or None
            Line dash dot (1-dot chain) setting to set.

        References
        ----------
        - GraphicsBase line_dash_dot_setting interface
            - https://simon-ritchie.github.io/apysc/en/graphics_base_line_dash_dot_setting.html  # noqa
        """
        self._update_line_dash_dot_setting_and_skip_appending_exp(value=value)
        self._append_line_dash_dot_setting_update_expression()

    @final
    def _update_line_dash_dot_setting_and_skip_appending_exp(
        self, *, value: Optional[LineDashDotSetting]
    ) -> None:
        """
        Update line dash-dot (1-dot chain) setting and skip
        appending expression.

        Parameters
        ----------
        value : LineDashDotSetting or None
            Line dash dot (1-dot chain) setting to set.
        """
        if value is not None and not isinstance(value, LineDashDotSetting):
            raise TypeError(
                "Not supported line_dash_dot_setting type specified: "
                f"{type(value)}"
                "\nAcceptable ones are: LineDashDotSetting or None."
            )
        self._line_dash_dot_setting = value

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_line_dash_dot_setting_update_expression(self) -> None:
        """
        Append line dash-dot setting updating expression.
        """
        import apysc as ap

        if self._line_dash_dot_setting is None:
            setting_str: str = '""'
        else:
            dot_size_name: str = self._line_dash_dot_setting.dot_size.variable_name
            dash_size_name: str = self._line_dash_dot_setting.dash_size.variable_name
            space_size_name: str = self._line_dash_dot_setting.space_size.variable_name
            setting_str = (
                f'String({dot_size_name}) + " " + '
                f'String({space_size_name}) + " " + '
                f'String({dash_size_name}) + " " + '
                f"String({space_size_name})"
            )
        expression: str = (
            f'{self.variable_name}.css("stroke-dasharray", {setting_str});'
        )
        ap.append_js_expression(expression=expression)

    _line_dash_dot_setting_snapshots: Dict[str, Optional[LineDashDotSetting]]

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make value's snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_line_dash_dot_setting_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name="_line_dash_dot_setting_snapshots",
            value=self._line_dash_dot_setting,
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert value if snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        if not self._snapshot_exists(snapshot_name=snapshot_name):
            return
        self._line_dash_dot_setting = self._line_dash_dot_setting_snapshots[
            snapshot_name
        ]



"""This module is for the translation mapping data of the
following document:

Document file: num_children.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# num_children interface": "# num_children インターフェイス",
    ##################################################
    "This page explains the container class, like the `Graphics`\\, `Sprite`\\, `Stage`) `num_children` property interface.": "このページでは`Graphics`や`Sprite`、`Stage`などのコンテナのクラスの`num_children`属性のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `num_children` property interface returns the integer (`Int`) value of the number of children.": "`num_children`属性のインターフェイスは子の数の`Int`型の整数の値を返却します。",  # noqa
    ##################################################
    "## Notes": "## 特記事項",
    ##################################################
    "The `Sprite` instance's initial children number is 1, not 0 since a sprite instance has a `graphics` child.": "`Sprite`インスタンスの初期値は`graphics`インスタンスの子を持つため0ではなく1になっています。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `num_children` property returns the number of children (`Int` value). You can use it for the calculation, for instance, coordinates calculation.": "`num_children`属性は`Int`型の整数の子の数を返却します。その値を使って座標の計算などを行うことができます。",  # noqa
    ##################################################
    "The following example appends a new rectangle when you click the sprite (rectangle) instance. The `num_children` property determines a new rectangle x-coordinate. When clicking a rectangle, this code also displays the current `num_children` property value to the browser console (please press the F12 key).": "以下のコード例では四角をクリックした際に新しい四角を追加しています。`num_children`属性の値は新しい四角のX座標を決めるのに使われています。また、このコードではクリックされた際に現在の`num_children`属性の値をブラウザのコンソールに表示しています（F12キーを押してコンソールを開いて確認してください）。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_sprite_click(e: ap.MouseEvent[ap.Sprite], options: dict) -> None:\n    """\n    The handler that the sprite calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    sprite: ap.Sprite = e.this\n    rectangle_x: ap.Int = (sprite.num_children - 1) * 100 + 50\n    new_rect: ap.Rectangle = sprite.graphics.draw_rect(\n        x=rectangle_x, y=50, width=50, height=50\n    )\n    sprite.add_child(new_rect)\n    ap.trace(\n        "Current sprite children number:",\n        sprite.num_children,\n        "rectangle x:",\n        rectangle_x,\n    )\n\n\nap.Stage(\n    background_color="#333", stage_width=450, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nsprite.add_child(rectangle_1)\nsprite.click(on_sprite_click)\n\nap.save_overall_html(dest_dir_path="num_children_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_sprite_click(e: ap.MouseEvent[ap.Sprite], options: dict) -> None:\n    """\n    The handler that the sprite calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    sprite: ap.Sprite = e.this\n    rectangle_x: ap.Int = (sprite.num_children - 1) * 100 + 50\n    new_rect: ap.Rectangle = sprite.graphics.draw_rect(\n        x=rectangle_x, y=50, width=50, height=50\n    )\n    sprite.add_child(new_rect)\n    ap.trace(\n        "Current sprite children number:",\n        sprite.num_children,\n        "rectangle x:",\n        rectangle_x,\n    )\n\n\nap.Stage(\n    background_color="#333", stage_width=450, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nsprite.add_child(rectangle_1)\nsprite.click(on_sprite_click)\n\nap.save_overall_html(dest_dir_path="num_children_basic_usage/")\n```',  # noqa
    ##################################################
    "## num_children API": "## num_children API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a current children's number.<hr>": "現在の子の数を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `num_children`: int": "- `num_children`: int",
    ##################################################
    "  - Current children number.": "  - 現在の子の数。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af", alpha=0.5)\n>>> rectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> rectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=150, y=50, width=50, height=50\n... )\n>>> sprite.graphics.num_children\nInt(2)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af", alpha=0.5)\n>>> rectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> rectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=150, y=50, width=50, height=50\n... )\n>>> sprite.graphics.num_children\nInt(2)\n```',  # noqa
}



"""
This module is for the lower scale limit's setter interface class.
"""

from typing_extensions import final

from apysc._type.number import Number

MIN_SCALE: float = 0.00000001


class SetLowerScaleLimitMixIn:
    @final
    def _set_lower_scale_limit(self, *, value: Number) -> None:
        """
        Set the lower scale limit to a specified value.

        Parameters
        ----------
        value : Number
            A value to apply the lower scale limit.
        """
        import apysc as ap

        with ap.If(value < MIN_SCALE, locals_=locals()):
            value.value = ap.Number(MIN_SCALE)
